/**
 * Image processing methods for iOS check deposit.
 * This requires the image rotation native module as well as
 * Appcelerator's ImageFactory module.
 */
var ImageFactory = require('ti.imagefactory');
var ImageRotationModule = require('com.mfoundry.mb.imagerotate');

/**
 * Rotates an image captured from the camera to landscape orientation.
 * This only works on iOS, not Android.
 */
var rotateImage = function(image) {
    return ImageRotationModule.rotateImage(image);
};

/**
 * Process a captured image to the required image quality settings.
 * This only works on iOS, not Android.
 * 
 * @param {Object} capturedImage
 *      A Ti.Blob containing the check image captured using the camera.
 * @param {Number} width
 *      Resize width, in pixels.
 * @param {Number} compression
 *      Compression, a float between 0 (low compression, high quality) and 1 (high compression, low quality)
 * @param {Boolean} color
 *      True for color, false for grayscale. This just affects the full sized check image for upload;
 *      the thumbnail will always be in color.
 * @param {Object} onSuccess
 *      Method to call when image processing is completed. Takes two parameters,
 *      a Ti.Blob containing the processed check image for upload to the server,
 *      and another Ti.Blob containing a thumbnail for display in the UI.
 * @param {Object} onError
 *      Method to call when image processing fails for some reason. Takes one
 *      parameter, an error message.
 * @param {View} helperView
 *      A View object that is currently visible. (The grayscale conversion
 *      requires a reference to a View.)
 */
var processCheckImage = function(capturedImage, originalImage, width, compression, color, onSuccess, onError, helperView) {
    
    var processedImage, thumbnailImage;
    
    // calculate the height and aspect ratio for resizing
    var aspectRatio = capturedImage.width / capturedImage.height;
    if (aspectRatio < 1) {
        aspectRatio = 1 / aspectRatio;
    }
    var height = width / aspectRatio;
    
    // Create a small thumbnail image
    // The thumbnail is half the width of the display and compressed to 0.5
    var thumbnailWidth  = Ti.Platform.displayCaps.platformWidth / 2;
    var thumbnailHeight = thumbnailWidth / aspectRatio;
    var THUMBNAIL_COMPRESSION = 0.5;
    thumbnailImage = capturedImage;
    thumbnailImage = resizeImage(thumbnailImage, thumbnailWidth, thumbnailHeight);
    thumbnailImage = ImageFactory.compress(thumbnailImage, THUMBNAIL_COMPRESSION);
    
    // Process the check image for upload to the image processor.
    // Grayscale conversion is asynchronous, so this is split into 2 steps,
    // "processImage1" and "processImage2".
    var processImage1 = function() {
        processedImage = capturedImage;
        processedImage = resizeImage(processedImage, width, height);
        // If using color images, run step 2 immediately,
        // otherwise start converting to grayscale and step 2
        // will run when it is finished.
        if (!color) {
            convertImageToGrayscale(
                processedImage,
                function(grayImage) {
                    processedImage = grayImage;
                    processImage2();
                },
                function() {
                    onError("Error converting to grayscale");
                },
                helperView
            );
        } else {
            processImage2();
        }
    };
    var processImage2 = function() {
		// NB, ImageFactory's compression is: 0=maximum, 1=none. It's the reverse of GBAPI.
        processedImage = ImageFactory.compress(processedImage, 1.0 - compression);
        // done
        Ti.API.debug("created processed image of length: " + processedImage.getLength());
        Ti.API.debug("created thumbnail image of length: " + thumbnailImage.getLength());
        onSuccess(processedImage, originalImage, thumbnailImage);
    };
    processImage1();
};

/**
 * 
 * @param {Object} imageBlob image to resize
 * @param width the target width
 * @param height the target height
 * @return the resized image
 */
function resizeImage(imageBlob, width, height) {
    return ImageFactory.imageAsResized(
        imageBlob,
        {
            width: width,
            height: height,
            format: ImageFactory.JPEG,
            quality: 1.0
        }
    );
}

/**
 * 
 * @param {Blob} imageBlob
 *      The image to convert to grayscale.
 * @param {Function} successCallback
 *      Called when grayscale conversion finishes successfully;
 *      receives one argument, a Blob containing the image.
 * @param {Function} errorCallback
 *      Called if grayscale conversion fails; receives no arguments.
 * @param {View} helperView
 *      A View object that is currently visible. (The grayscale conversion
 *      requires a reference to a View.)
 */
function convertImageToGrayscale(imageBlob, successCallback, errorCallback, helperView) {
    var dataUriPrefix = "data:image/jpeg;base64,";

    var W = imageBlob.width;
    var H = imageBlob.height;
    
    var html = '<html><head></head><body><canvas id="myCanvas" width="'+W+'" height="'+H+'"></canvas><img id="myImg" src="' + dataUriPrefix + Ti.Utils.base64encode(imageBlob) + '"></img></body></html>';

    var js = '(function(){'
        + 'var canvas=document.getElementById("myCanvas");'
        + 'var canvasContext=canvas.getContext("2d");'
        + 'var img=document.getElementById("myImg");'
        + '    canvasContext.drawImage(img, 0, 0, ' + W + ', ' + H + ');'
        + '    var imgPixels = canvasContext.getImageData(0, 0, ' + W + ', ' + H + ');'
        + '    for(var y = 0; y < imgPixels.height; y++) { for(var x = 0; x < imgPixels.width; x++) {var i = (y * 4) * imgPixels.width + x * 4;var avg = (imgPixels.data[i] + imgPixels.data[i + 1] + imgPixels.data[i + 2]) / 3;imgPixels.data[i] = avg;imgPixels.data[i + 1] = avg;imgPixels.data[i + 2] = avg;}}'
        + '    canvasContext.putImageData(imgPixels, 0, 0);'
        + '    return canvas.toDataURL("image/jpeg", 1.0);'
        + '})();';

    // Position the webview off the parent image. Ti now clips on parent boundaries so it should
	// be enough to push it off the parent and not need to push it off device. Specifying a size
	// of 0,0 is not enough to make it hidden.
    var webView = Ti.UI.createWebView({
        html: html,
		top: -100,
		left: -100,
        width: 0,
        height: 0
    });
    
    var finished = false;
    
    var grayscaleImageDataUri = null;
    webView.addEventListener('load', function() {
        grayscaleImageDataUri = webView.evalJS(js);
        // alert("[" + grayscaleImageDataUri.length + "]" + grayscaleImageDataUri.substring(0,30) + "...");
        var grayscaleImageBase64 = grayscaleImageDataUri.substring(grayscaleImageDataUri.indexOf(',')+1);
        // alert(grayscaleImageBase64.substring(0,20));
        if (!finished) {
            finished = true;
            helperView.remove(webView);
            successCallback(Ti.Utils.base64decode(grayscaleImageBase64));
        }
    });
    
    // add the webview to the helper view to make it load the html
    helperView.add(webView);
    
    // if grayscale conversion doesn't finish within 20 seconds
    // there was some sort of error within the webview
    setTimeout(function() {
        if (!finished && grayscaleImageDataUri === null) {
            finished = true;
            helperView.remove(webView);
            errorCallback();
        }
    }, 20000);
}

// EXPORTS

exports.rotateImage = rotateImage;
exports.processCheckImage = processCheckImage;
